@@ -1,4 +1,4 @@
-/* The GIMP -- an image manipulation program
+/* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995-2001 Spencer Kimball, Peter Mattis and others
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,21 +21,23 @@
 #include <gtk/gtk.h>
 
 #include "libgimpbase/gimpbase.h"
+#include "libgimpconfig/gimpconfig.h"
+#include "libgimpwidgets/gimpwidgets.h"
 
 #include "tools-types.h"
 
-#include "config/gimpconfig.h"
-
 #include "core/gimp.h"
-#include "core/gimplist.h"
+#include "core/gimp-contexts.h"
 #include "core/gimptoolinfo.h"
 #include "core/gimptooloptions.h"
+#include "core/gimptoolpresets.h"
 
 #include "gimp-tools.h"
 #include "gimptooloptions-gui.h"
 #include "tool_manager.h"
 
 #include "gimpairbrushtool.h"
+#include "gimpaligntool.h"
 #include "gimpblendtool.h"
 #include "gimpbrightnesscontrasttool.h"
 #include "gimpbucketfilltool.h"
@@ -52,7 +54,9 @@
 #include "gimperasertool.h"
 #include "gimpfliptool.h"
 #include "gimpfreeselecttool.h"
+#include "gimpforegroundselecttool.h"
 #include "gimpfuzzyselecttool.h"
+#include "gimphealtool.h"
 #include "gimphuesaturationtool.h"
 #include "gimpinktool.h"
 #include "gimpiscissorstool.h"
@@ -62,10 +66,11 @@
 #include "gimpmovetool.h"
 #include "gimppaintbrushtool.h"
 #include "gimppenciltool.h"
+#include "gimpperspectiveclonetool.h"
 #include "gimpperspectivetool.h"
 #include "gimpposterizetool.h"
-#include "gimprectselecttool.h"
 #include "gimpthresholdtool.h"
+#include "gimprectangleselecttool.h"
 #include "gimprotatetool.h"
 #include "gimpscaletool.h"
 #include "gimpsheartool.h"
@@ -93,6 +98,11 @@ static void   gimp_tools_register (GType                   tool_type,
                                    gpointer                data);
 
 
+/*  private variables  */
+
+static gboolean   tool_options_deleted = FALSE;
+
+
 /*  public functions  */
 
 void
@@ -117,6 +127,8 @@ gimp_tools_init (Gimp *gimp)
     gimp_dodge_burn_tool_register,
     gimp_smudge_tool_register,
     gimp_convolve_tool_register,
+    gimp_perspective_clone_tool_register,
+    gimp_heal_tool_register,
     gimp_clone_tool_register,
     gimp_ink_tool_register,
     gimp_airbrush_tool_register,
@@ -135,6 +147,7 @@ gimp_tools_init (Gimp *gimp)
     gimp_scale_tool_register,
     gimp_rotate_tool_register,
     gimp_crop_tool_register,
+    gimp_align_tool_register,
     gimp_move_tool_register,
 
     /*  non-modifying tools  */
@@ -149,6 +162,7 @@ gimp_tools_init (Gimp *gimp)
 
     /*  selection tools */
 
+    gimp_foreground_select_tool_register,
     gimp_iscissors_tool_register,
     gimp_by_color_select_tool_register,
     gimp_fuzzy_select_tool_register,
@@ -185,7 +199,8 @@ gimp_tools_init (Gimp *gimp)
 
   default_order = g_list_reverse (default_order);
 
-  g_object_set_data (G_OBJECT (gimp), "gimp-tools-default-order", default_order);
+  g_object_set_data (G_OBJECT (gimp),
+                     "gimp-tools-default-order", default_order);
 }
 
 void
@@ -219,9 +234,13 @@ gimp_tools_restore (Gimp *gimp)
 
   filename = gimp_personal_rc_file ("toolrc");
 
+  if (gimp->be_verbose)
+    g_print ("Parsing '%s'\n", gimp_filename_to_utf8 (filename));
+
   if (gimp_config_deserialize_file (GIMP_CONFIG (gimp_list), filename,
                                     NULL, NULL))
     {
+      gint n = gimp_container_num_children (gimp->tool_info_list);
       gint i;
 
       gimp_list_reverse (GIMP_LIST (gimp_list));
@@ -244,7 +263,8 @@ gimp_tools_restore (Gimp *gimp)
                             "visible", GIMP_TOOL_INFO (list->data)->visible,
                             NULL);
 
-              gimp_container_reorder (gimp->tool_info_list, object, i);
+              gimp_container_reorder (gimp->tool_info_list,
+                                      object, MIN (i, n - 1));
             }
         }
     }
@@ -256,16 +276,27 @@ gimp_tools_restore (Gimp *gimp)
        list;
        list = g_list_next (list))
     {
-      GimpToolInfo           *tool_info;
-      GimpToolOptionsGUIFunc  options_gui_func;
-      GtkWidget              *options_gui;
-
-      tool_info = GIMP_TOOL_INFO (list->data);
+      GimpToolInfo *tool_info = GIMP_TOOL_INFO (list->data);
 
       /*  get default values from prefs (see bug #120832)  */
       gimp_tool_options_reset (tool_info->tool_options);
+    }
 
-      gimp_tool_options_deserialize (tool_info->tool_options, NULL, NULL);
+  gimp_contexts_load (gimp);
+
+  for (list = GIMP_LIST (gimp->tool_info_list)->list;
+       list;
+       list = g_list_next (list))
+    {
+      GimpToolInfo           *tool_info = GIMP_TOOL_INFO (list->data);
+      GimpToolOptionsGUIFunc  options_gui_func;
+      GtkWidget              *options_gui;
+
+      gimp_context_copy_properties (gimp_get_user_context (gimp),
+                                    GIMP_CONTEXT (tool_info->tool_options),
+                                    GIMP_CONTEXT_ALL_PROPS_MASK);
+
+      gimp_tool_options_deserialize (tool_info->tool_options, NULL);
 
       options_gui_func = g_object_get_data (G_OBJECT (tool_info),
                                             "gimp-tool-options-gui-func");
@@ -280,7 +311,11 @@ gimp_tools_restore (Gimp *gimp)
 
           options_gui = gimp_tool_options_gui (tool_info->tool_options);
 
-          label = gtk_label_new (_("This tool has no options."));
+          label = gtk_label_new (_("This tool has\nno options."));
+          gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_CENTER);
+          gimp_label_set_attributes (GTK_LABEL (label),
+                                     PANGO_ATTR_STYLE, PANGO_STYLE_ITALIC,
+                                     -1);
           gtk_box_pack_start (GTK_BOX (options_gui), label, FALSE, FALSE, 6);
           gtk_widget_show (label);
         }
@@ -288,71 +323,43 @@ gimp_tools_restore (Gimp *gimp)
       g_object_set_data (G_OBJECT (tool_info->tool_options),
                          "gimp-tool-options-gui", options_gui);
 
-      if (tool_info->options_presets)
-        {
-          gchar *filename;
-          GList *list;
-
-          filename = gimp_tool_options_build_filename (tool_info->tool_options,
-                                                       "presets");
-          gimp_config_deserialize_file (GIMP_CONFIG (tool_info->options_presets),
-                                        filename,
-                                        gimp, NULL);
-          g_free (filename);
-
-          gimp_list_reverse (GIMP_LIST (tool_info->options_presets));
-
-          for (list = GIMP_LIST (tool_info->options_presets)->list;
-               list;
-               list = g_list_next (list))
-            {
-              g_object_set (list->data, "tool-info", tool_info, NULL);
-            }
-        }
+      if (tool_info->presets)
+        gimp_tool_presets_load (tool_info->presets, NULL);
     }
 }
 
 void
-gimp_tools_save (Gimp *gimp)
+gimp_tools_save (Gimp     *gimp,
+                 gboolean  save_tool_options,
+                 gboolean  always_save)
 {
-  GList *list;
   gchar *filename;
 
   g_return_if_fail (GIMP_IS_GIMP (gimp));
 
-  for (list = GIMP_LIST (gimp->tool_info_list)->list;
-       list;
-       list = g_list_next (list))
+  if (save_tool_options && (! tool_options_deleted || always_save))
     {
-      GimpToolInfo *tool_info = GIMP_TOOL_INFO (list->data);
+      GList *list;
 
-      gimp_tool_options_serialize (tool_info->tool_options, NULL, NULL);
+      gimp_contexts_save (gimp);
 
-      if (tool_info->options_presets)
+      gimp_tool_options_create_folder ();
+
+      for (list = GIMP_LIST (gimp->tool_info_list)->list;
+           list;
+           list = g_list_next (list))
         {
-          gchar *filename;
-          gchar *header;
-          gchar *footer;
+          GimpToolInfo *tool_info = GIMP_TOOL_INFO (list->data);
 
-          filename = gimp_tool_options_build_filename (tool_info->tool_options,
-                                                       "presets");
-
-          header = g_strdup_printf ("GIMP %s options presets",
-                                    GIMP_OBJECT (tool_info)->name);
-          footer = g_strdup_printf ("end of %s options presets",
-                                    GIMP_OBJECT (tool_info)->name);
-
-          gimp_config_serialize_to_file (GIMP_CONFIG (tool_info->options_presets),
-                                         filename, header, footer,
-                                         NULL, NULL);
-
-          g_free (filename);
-          g_free (header);
-          g_free (footer);
+          gimp_tool_options_serialize (tool_info->tool_options, NULL);
         }
     }
 
   filename = gimp_personal_rc_file ("toolrc");
+
+  if (gimp->be_verbose)
+    g_print ("Writing '%s'\n", gimp_filename_to_utf8 (filename));
+
   gimp_config_serialize_to_file (GIMP_CONFIG (gimp->tool_info_list),
                                  filename,
                                  "GIMP toolrc",
@@ -361,6 +368,33 @@ gimp_tools_save (Gimp *gimp)
   g_free (filename);
 }
 
+gboolean
+gimp_tools_clear (Gimp    *gimp,
+                  GError **error)
+{
+  GList    *list;
+  gboolean  success = TRUE;
+
+  g_return_val_if_fail (GIMP_IS_GIMP (gimp), FALSE);
+
+  for (list = GIMP_LIST (gimp->tool_info_list)->list;
+       list && success;
+       list = g_list_next (list))
+    {
+      GimpToolInfo *tool_info = GIMP_TOOL_INFO (list->data);
+
+      success = gimp_tool_options_delete (tool_info->tool_options, NULL);
+    }
+
+  if (success)
+    success = gimp_contexts_clear (gimp, error);
+
+  if (success)
+    tool_options_deleted = TRUE;
+
+  return success;
+}
+
 GList *
 gimp_tools_get_default_order (Gimp *gimp)
 {
@@ -391,6 +425,7 @@ gimp_tools_register (GType                   tool_type,
   Gimp         *gimp = (Gimp *) data;
   GimpToolInfo *tool_info;
   const gchar  *paint_core_name;
+  gboolean      visible;
 
   g_return_if_fail (GIMP_IS_GIMP (gimp));
   g_return_if_fail (g_type_is_a (tool_type, GIMP_TYPE_TOOL));
@@ -402,61 +437,72 @@ gimp_tools_register (GType                   tool_type,
 
   if (tool_type == GIMP_TYPE_PENCIL_TOOL)
     {
-      paint_core_name = "GimpPencil";
+      paint_core_name = "gimp-pencil";
     }
   else if (tool_type == GIMP_TYPE_PAINTBRUSH_TOOL)
     {
-      paint_core_name = "GimpPaintbrush";
+      paint_core_name = "gimp-paintbrush";
     }
   else if (tool_type == GIMP_TYPE_ERASER_TOOL)
     {
-      paint_core_name = "GimpEraser";
+      paint_core_name = "gimp-eraser";
     }
   else if (tool_type == GIMP_TYPE_AIRBRUSH_TOOL)
     {
-      paint_core_name = "GimpAirbrush";
+      paint_core_name = "gimp-airbrush";
     }
   else if (tool_type == GIMP_TYPE_CLONE_TOOL)
     {
-      paint_core_name = "GimpClone";
+      paint_core_name = "gimp-clone";
+    }
+  else if (tool_type == GIMP_TYPE_HEAL_TOOL)
+    {
+      paint_core_name = "gimp-heal";
+    }
+  else if (tool_type == GIMP_TYPE_PERSPECTIVE_CLONE_TOOL)
+    {
+      paint_core_name = "gimp-perspective-clone";
     }
   else if (tool_type == GIMP_TYPE_CONVOLVE_TOOL)
     {
-      paint_core_name = "GimpConvolve";
+      paint_core_name = "gimp-convolve";
     }
   else if (tool_type == GIMP_TYPE_SMUDGE_TOOL)
     {
-      paint_core_name = "GimpSmudge";
+      paint_core_name = "gimp-smudge";
     }
   else if (tool_type == GIMP_TYPE_DODGE_BURN_TOOL)
     {
-      paint_core_name = "GimpDodgeBurn";
+      paint_core_name = "gimp-dodge-burn";
     }
   else if (tool_type == GIMP_TYPE_INK_TOOL)
     {
-      paint_core_name = "GimpInk";
+      paint_core_name = "gimp-ink";
     }
   else
     {
-      paint_core_name = "GimpPaintbrush";
+      paint_core_name = "gimp-paintbrush";
     }
 
   tool_info = gimp_tool_info_new (gimp,
-				  tool_type,
+                                  tool_type,
                                   tool_options_type,
-				  context_props,
-				  identifier,
-				  blurb,
-				  help,
-				  menu_path,
-				  menu_accel,
-				  help_domain,
-				  help_data,
+                                  context_props,
+                                  identifier,
+                                  blurb,
+                                  help,
+                                  menu_path,
+                                  menu_accel,
+                                  help_domain,
+                                  help_data,
                                   paint_core_name,
-				  stock_id);
+                                  stock_id);
 
-  if (g_type_is_a (tool_type, GIMP_TYPE_IMAGE_MAP_TOOL))
-    g_object_set (tool_info, "visible", FALSE, NULL);
+  visible = (! g_type_is_a (tool_type, GIMP_TYPE_IMAGE_MAP_TOOL));
+
+  g_object_set (tool_info, "visible", visible, NULL);
+  g_object_set_data (G_OBJECT (tool_info), "gimp-tool-default-visible",
+                     GINT_TO_POINTER (visible));
 
   g_object_set_data (G_OBJECT (tool_info), "gimp-tool-options-gui-func",
                      options_gui_func);
