@@ -1,8 +1,8 @@
-/* The GIMP -- an image manipulation program
+/* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
  *
  * gimpactionview.c
- * Copyright (C) 2004  Michael Natterer <mitch@gimp.org>
+ * Copyright (C) 2004-2005  Michael Natterer <mitch@gimp.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,23 +30,20 @@
 
 #include "widgets-types.h"
 
+#include "core/gimp.h"
+
 #include "gimpaction.h"
 #include "gimpactiongroup.h"
 #include "gimpactionview.h"
-#include "gimpcellrendereraccel.h"
 #include "gimpmessagebox.h"
 #include "gimpmessagedialog.h"
 #include "gimpuimanager.h"
-#include "gimpwidgets-utils.h"
 
 #include "gimp-intl.h"
 
 
 /*  local function prototypes  */
 
-static void     gimp_action_view_class_init  (GimpActionViewClass *klass);
-static void     gimp_action_view_init        (GimpActionView      *view);
-
 static void     gimp_action_view_dispose         (GObject         *object);
 
 static gboolean gimp_action_view_accel_find_func (GtkAccelKey     *key,
@@ -57,52 +54,27 @@ static void     gimp_action_view_accel_changed   (GtkAccelGroup   *accel_group,
                                                   GdkModifierType  unused2,
                                                   GClosure        *accel_closure,
                                                   GimpActionView  *view);
-static void     gimp_action_view_accel_edited    (GimpCellRendererAccel *accel,
+static void     gimp_action_view_accel_edited    (GtkCellRendererAccel *accel,
                                                   const char      *path_string,
-                                                  gboolean         delete,
                                                   guint            accel_key,
                                                   GdkModifierType  accel_mask,
+                                                  guint            hardware_keycode,
+                                                  GimpActionView  *view);
+static void     gimp_action_view_accel_cleared   (GtkCellRendererAccel *accel,
+                                                  const char      *path_string,
                                                   GimpActionView  *view);
 
 
-static GtkTreeViewClass *parent_class = NULL;
+G_DEFINE_TYPE (GimpActionView, gimp_action_view, GTK_TYPE_TREE_VIEW)
 
+#define parent_class gimp_action_view_parent_class
 
-GType
-gimp_action_view_get_type (void)
-{
-  static GType view_type = 0;
-
-  if (! view_type)
-    {
-      static const GTypeInfo view_info =
-      {
-        sizeof (GimpActionViewClass),
-        NULL,           /* base_init      */
-        NULL,           /* base_finalize  */
-        (GClassInitFunc) gimp_action_view_class_init,
-        NULL,           /* class_finalize */
-        NULL,           /* class_data     */
-        sizeof (GimpActionView),
-        0,              /* n_preallocs    */
-        (GInstanceInitFunc) gimp_action_view_init
-      };
-
-      view_type = g_type_register_static (GTK_TYPE_TREE_VIEW,
-                                          "GimpActionView",
-                                          &view_info, 0);
-    }
-
-  return view_type;
-}
 
 static void
 gimp_action_view_class_init (GimpActionViewClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-  parent_class = g_type_class_peek_parent (klass);
-
   object_class->dispose = gimp_action_view_dispose;
 }
 
@@ -169,8 +141,8 @@ gimp_action_view_button_press (GtkWidget      *widget,
 
   if (gtk_tree_view_get_path_at_pos (tree_view,
                                      (gint) event->x,
-				     (gint) event->y,
-				     &path, NULL,
+                                     (gint) event->y,
+                                     &path, NULL,
                                      NULL, NULL))
     {
       GClosure *closure;
@@ -178,14 +150,14 @@ gimp_action_view_button_press (GtkWidget      *widget,
 
       if (gtk_tree_path_get_depth (path) == 1)
         {
-	  gtk_tree_path_free (path);
-	  return FALSE;
-	}
+          gtk_tree_path_free (path);
+          return FALSE;
+        }
 
       g_object_set_data_full (G_OBJECT (tree_view), "start_editing_path",
                               path, (GDestroyNotify) gtk_tree_path_free);
 
-      g_signal_stop_emission_by_name (tree_view, "button_press_event");
+      g_signal_stop_emission_by_name (tree_view, "button-press-event");
 
       closure = g_cclosure_new_object (G_CALLBACK (idle_start_editing),
                                        G_OBJECT (tree_view));
@@ -193,6 +165,7 @@ gimp_action_view_button_press (GtkWidget      *widget,
       source = g_idle_source_new ();
       g_source_set_closure (source, closure);
       g_source_attach (source, NULL);
+      g_source_unref (source);
     }
 
   return TRUE;
@@ -214,13 +187,13 @@ gimp_action_view_new (GimpUIManager *manager,
   g_return_val_if_fail (GIMP_IS_UI_MANAGER (manager), NULL);
 
   store = gtk_tree_store_new (GIMP_ACTION_VIEW_NUM_COLUMNS,
-                              GTK_TYPE_ACTION,        /* COLUMN_ACTION     */
-                              G_TYPE_STRING,          /* COLUMN_STOCK_ID   */
-                              G_TYPE_STRING,          /* COLUMN_LABEL      */
-                              G_TYPE_STRING,          /* COLUMN_NAME       */
-                              G_TYPE_UINT,            /* COLUMN_ACCEL_KEY  */
-                              GDK_TYPE_MODIFIER_TYPE, /* COLUMN_ACCEL_MASK */
-                              GTK_TYPE_MENU_ITEM);    /* COLUMN_MENU_ITEM  */
+                              GTK_TYPE_ACTION,        /* COLUMN_ACTION        */
+                              G_TYPE_STRING,          /* COLUMN_STOCK_ID      */
+                              G_TYPE_STRING,          /* COLUMN_LABEL         */
+                              G_TYPE_STRING,          /* COLUMN_NAME          */
+                              G_TYPE_UINT,            /* COLUMN_ACCEL_KEY     */
+                              GDK_TYPE_MODIFIER_TYPE, /* COLUMN_ACCEL_MASK    */
+                              G_TYPE_CLOSURE);        /* COLUMN_ACCEL_CLOSURE */
 
   accel_group = gtk_ui_manager_get_accel_group (GTK_UI_MANAGER (manager));
 
@@ -246,95 +219,77 @@ gimp_action_view_new (GimpUIManager *manager,
 
       for (list2 = actions; list2; list2 = g_list_next (list2))
         {
-          GtkAction   *action = list2->data;
-          const gchar *name   = gtk_action_get_name (action);
+          GtkAction       *action        = list2->data;
+          const gchar     *name          = gtk_action_get_name (action);
+          gchar           *stock_id;
+          gchar           *label;
+          gchar           *tmp;
+          guint            accel_key     = 0;
+          GdkModifierType  accel_mask    = 0;
+          GClosure        *accel_closure = NULL;
+          GtkTreeIter      action_iter;
 
-          if (! strstr (name, "-menu") &&
-              ! strstr (name, "-popup"))
+          if (strstr (name, "-menu")  ||
+              strstr (name, "-popup") ||
+              name[0] == '<')
+            continue;
+
+          g_object_get (action,
+                        "stock-id", &stock_id,
+                        "label",    &tmp,
+                        NULL);
+
+          label = gimp_strip_uline (tmp);
+          g_free (tmp);
+
+          if (! (label && strlen (label)))
             {
-              GtkTreeIter      action_iter;
-              gchar           *stock_id;
-              gchar           *label;
-              gchar           *stripped;
-              guint            accel_key  = 0;
-              GdkModifierType  accel_mask = 0;
-              GtkWidget       *menu_item  = NULL;
-
-              g_object_get (action,
-                            "stock-id", &stock_id,
-                            "label",    &label,
-                            NULL);
-
-              stripped = gimp_strip_uline (label);
-
-              if (show_shortcuts)
-                {
-                  menu_item = gtk_action_create_menu_item (action);
-
-                  if (GTK_IS_MENU_ITEM (menu_item) &&
-                      GTK_IS_ACCEL_LABEL (GTK_BIN (menu_item)->child))
-                    {
-                      GtkWidget *accel_label = GTK_BIN (menu_item)->child;
-                      GClosure  *accel_closure;
-
-                      g_object_get (accel_label,
-                                    "accel-closure", &accel_closure,
-                                    NULL);
-
-                      if (accel_closure)
-                        {
-                          GtkAccelKey *key;
-
-                          key = gtk_accel_group_find (accel_group,
-                                                      gimp_action_view_accel_find_func,
-                                                      accel_closure);
-
-                          if (key            &&
-                              key->accel_key &&
-                              key->accel_flags & GTK_ACCEL_VISIBLE)
-                            {
-                              accel_key  = key->accel_key;
-                              accel_mask = key->accel_mods;
-                            }
-
-                          g_closure_unref (accel_closure);
-                        }
-
-                      g_object_ref (menu_item);
-                      gtk_object_sink (GTK_OBJECT (menu_item));
-                    }
-                  else if (menu_item)
-                    {
-                      gtk_object_sink (GTK_OBJECT (menu_item));
-                      menu_item = NULL;
-                    }
-                }
-
-              gtk_tree_store_append (store, &action_iter, &group_iter);
-
-              gtk_tree_store_set (store, &action_iter,
-                                  GIMP_ACTION_VIEW_COLUMN_ACTION,     action,
-                                  GIMP_ACTION_VIEW_COLUMN_STOCK_ID,   stock_id,
-                                  GIMP_ACTION_VIEW_COLUMN_LABEL,      stripped,
-                                  GIMP_ACTION_VIEW_COLUMN_NAME,       name,
-                                  GIMP_ACTION_VIEW_COLUMN_ACCEL_KEY,  accel_key,
-                                  GIMP_ACTION_VIEW_COLUMN_ACCEL_MASK, accel_mask,
-                                  GIMP_ACTION_VIEW_COLUMN_MENU_ITEM,  menu_item,
-                                  -1);
-
-              g_free (stock_id);
               g_free (label);
-              g_free (stripped);
+              label = g_strdup (name);
+            }
 
-              if (menu_item)
-                g_object_unref (menu_item);
+          if (show_shortcuts)
+            {
+              accel_closure = gtk_action_get_accel_closure (action);
 
-              if (select_action && ! strcmp (select_action, name))
+              if (accel_closure)
                 {
-                  select_path = gtk_tree_model_get_path (GTK_TREE_MODEL (store),
-                                                         &action_iter);
+                  GtkAccelKey *key;
+
+                  key = gtk_accel_group_find (accel_group,
+                                              gimp_action_view_accel_find_func,
+                                              accel_closure);
+
+                  if (key            &&
+                      key->accel_key &&
+                      key->accel_flags & GTK_ACCEL_VISIBLE)
+                    {
+                      accel_key  = key->accel_key;
+                      accel_mask = key->accel_mods;
+                    }
                 }
             }
+
+          gtk_tree_store_append (store, &action_iter, &group_iter);
+
+          gtk_tree_store_set (store, &action_iter,
+                              GIMP_ACTION_VIEW_COLUMN_ACTION,        action,
+                              GIMP_ACTION_VIEW_COLUMN_STOCK_ID,      stock_id,
+                              GIMP_ACTION_VIEW_COLUMN_LABEL,         label,
+                              GIMP_ACTION_VIEW_COLUMN_NAME,          name,
+                              GIMP_ACTION_VIEW_COLUMN_ACCEL_KEY,     accel_key,
+                              GIMP_ACTION_VIEW_COLUMN_ACCEL_MASK,    accel_mask,
+                              GIMP_ACTION_VIEW_COLUMN_ACCEL_CLOSURE, accel_closure,
+                              -1);
+
+          g_free (stock_id);
+          g_free (label);
+
+          if (select_action && ! strcmp (select_action, name))
+            {
+              select_path = gtk_tree_model_get_path (GTK_TREE_MODEL (store),
+                                                     &action_iter);
+            }
         }
 
       g_list_free (actions);
@@ -350,6 +305,9 @@ gimp_action_view_new (GimpUIManager *manager,
   GIMP_ACTION_VIEW (view)->manager        = g_object_ref (manager);
   GIMP_ACTION_VIEW (view)->show_shortcuts = show_shortcuts;
 
+  gtk_tree_view_set_search_column (GTK_TREE_VIEW (view),
+                                   GIMP_ACTION_VIEW_COLUMN_LABEL);
+
   column = gtk_tree_view_column_new ();
   gtk_tree_view_column_set_title (column, _("Action"));
 
@@ -371,7 +329,7 @@ gimp_action_view_new (GimpUIManager *manager,
 
   if (show_shortcuts)
     {
-      g_signal_connect (view, "button_press_event",
+      g_signal_connect (view, "button-press-event",
                         G_CALLBACK (gimp_action_view_button_press),
                         NULL);
 
@@ -382,37 +340,38 @@ gimp_action_view_new (GimpUIManager *manager,
       column = gtk_tree_view_column_new ();
       gtk_tree_view_column_set_title (column, _("Shortcut"));
 
-      cell = gimp_cell_renderer_accel_new ();
+      cell = gtk_cell_renderer_accel_new ();
       cell->mode = GTK_CELL_RENDERER_MODE_EDITABLE;
       GTK_CELL_RENDERER_TEXT (cell)->editable = TRUE;
       gtk_tree_view_column_pack_start (column, cell, TRUE);
       gtk_tree_view_column_set_attributes (column, cell,
                                            "accel-key",
                                            GIMP_ACTION_VIEW_COLUMN_ACCEL_KEY,
-                                           "accel-mask",
+                                           "accel-mods",
                                            GIMP_ACTION_VIEW_COLUMN_ACCEL_MASK,
                                            NULL);
 
       g_signal_connect (cell, "accel-edited",
                         G_CALLBACK (gimp_action_view_accel_edited),
                         view);
+      g_signal_connect (cell, "accel-cleared",
+                        G_CALLBACK (gimp_action_view_accel_cleared),
+                        view);
 
       gtk_tree_view_append_column (view, column);
     }
-  else
-    {
-      column = gtk_tree_view_column_new ();
-      gtk_tree_view_column_set_title (column, _("Name"));
 
-      cell = gtk_cell_renderer_text_new ();
-      gtk_tree_view_column_pack_start (column, cell, TRUE);
-      gtk_tree_view_column_set_attributes (column, cell,
-                                           "text",
-                                           GIMP_ACTION_VIEW_COLUMN_NAME,
-                                           NULL);
+  column = gtk_tree_view_column_new ();
+  gtk_tree_view_column_set_title (column, _("Name"));
 
-      gtk_tree_view_append_column (view, column);
-    }
+  cell = gtk_cell_renderer_text_new ();
+  gtk_tree_view_column_pack_start (column, cell, TRUE);
+  gtk_tree_view_column_set_attributes (column, cell,
+                                       "text",
+                                       GIMP_ACTION_VIEW_COLUMN_NAME,
+                                       NULL);
+
+  gtk_tree_view_append_column (view, column);
 
   if (select_path)
     {
@@ -426,7 +385,7 @@ gimp_action_view_new (GimpUIManager *manager,
       gtk_tree_selection_select_path (sel, select_path);
 
       gtk_tree_view_scroll_to_cell (view, select_path, NULL,
-                                    FALSE, 0.0, 0.0);
+                                    TRUE, 0.5, 0.0);
 
       gtk_tree_path_free (select_path);
       gtk_tree_path_free (expand);
@@ -473,49 +432,39 @@ gimp_action_view_accel_changed (GtkAccelGroup   *accel_group,
            child_valid;
            child_valid = gtk_tree_model_iter_next (model, &child_iter))
         {
-          GtkWidget *menu_item;
+          GClosure *closure;
 
           gtk_tree_model_get (model, &child_iter,
-                              GIMP_ACTION_VIEW_COLUMN_MENU_ITEM, &menu_item,
+                              GIMP_ACTION_VIEW_COLUMN_ACCEL_CLOSURE, &closure,
                               -1);
 
-          if (menu_item)
+          if (closure)
+            g_closure_unref (closure);
+
+          if (accel_closure == closure)
             {
-              GClosure *item_closure;
+              GtkAccelKey     *key;
+              guint            accel_key  = 0;
+              GdkModifierType  accel_mask = 0;
 
-              g_object_get (GTK_BIN (menu_item)->child,
-                            "accel-closure", &item_closure,
-                            NULL);
+              key = gtk_accel_group_find (accel_group,
+                                          gimp_action_view_accel_find_func,
+                                          accel_closure);
 
-              g_object_unref (menu_item);
-
-              if (accel_closure == item_closure)
+              if (key            &&
+                  key->accel_key &&
+                  key->accel_flags & GTK_ACCEL_VISIBLE)
                 {
-                  GtkAccelKey     *key;
-                  guint            accel_key  = 0;
-                  GdkModifierType  accel_mask = 0;
-
-                  key = gtk_accel_group_find (accel_group,
-                                              gimp_action_view_accel_find_func,
-                                              accel_closure);
-
-                  if (key            &&
-                      key->accel_key &&
-                      key->accel_flags & GTK_ACCEL_VISIBLE)
-                    {
-                      accel_key  = key->accel_key;
-                      accel_mask = key->accel_mods;
-                    }
-
-                  gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
-                                      GIMP_ACTION_VIEW_COLUMN_ACCEL_KEY,
-                                      accel_key,
-                                      GIMP_ACTION_VIEW_COLUMN_ACCEL_MASK,
-                                      accel_mask,
-                                      -1);
-
-                  return;
+                  accel_key  = key->accel_key;
+                  accel_mask = key->accel_mods;
                 }
+
+              gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
+                                  GIMP_ACTION_VIEW_COLUMN_ACCEL_KEY,  accel_key,
+                                  GIMP_ACTION_VIEW_COLUMN_ACCEL_MASK, accel_mask,
+                                  -1);
+
+              return;
             }
         }
     }
@@ -523,6 +472,7 @@ gimp_action_view_accel_changed (GtkAccelGroup   *accel_group,
 
 typedef struct
 {
+  GimpUIManager   *manager;
   gchar           *accel_path;
   guint            accel_key;
   GdkModifierType  accel_mask;
@@ -542,12 +492,15 @@ gimp_action_view_conflict_response (GtkWidget   *dialog,
                                         confirm_data->accel_mask,
                                         TRUE))
         {
-          g_message (_("Changing shortcut failed."));
+          gimp_message (confirm_data->manager->gimp, G_OBJECT (dialog),
+                        GIMP_MESSAGE_ERROR,
+                        _("Changing shortcut failed."));
         }
     }
 
   g_free (confirm_data->accel_path);
-  g_free (confirm_data);
+
+  g_slice_free (ConfirmData, confirm_data);
 }
 
 static void
@@ -558,25 +511,26 @@ gimp_action_view_conflict_confirm (GimpActionView  *view,
                                    const gchar     *accel_path)
 {
   GimpActionGroup *group;
-  GimpMessageBox  *box;
   gchar           *label;
-  gchar           *stripped;
+  gchar           *tmp;
   gchar           *accel_string;
   ConfirmData     *confirm_data;
   GtkWidget       *dialog;
+  GimpMessageBox  *box;
 
   g_object_get (action,
                 "action-group", &group,
-                "label",        &label,
+                "label",        &tmp,
                 NULL);
 
-  stripped = gimp_strip_uline (label);
-  g_free (label);
+  label = gimp_strip_uline (tmp);
+  g_free (tmp);
 
-  accel_string = gimp_get_accel_string (accel_key, accel_mask);
+  accel_string = gtk_accelerator_get_label (accel_key, accel_mask);
 
-  confirm_data = g_new0 (ConfirmData, 1);
+  confirm_data = g_slice_new (ConfirmData);
 
+  confirm_data->manager    = view->manager;
   confirm_data->accel_path = g_strdup (accel_path);
   confirm_data->accel_key  = accel_key;
   confirm_data->accel_mask = accel_mask;
@@ -592,6 +546,10 @@ gimp_action_view_conflict_confirm (GimpActionView  *view,
 
                              NULL);
 
+  gtk_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
+                                           GTK_RESPONSE_OK,
+                                           GTK_RESPONSE_CANCEL,
+                                           -1);
   g_signal_connect (dialog, "response",
                     G_CALLBACK (gimp_action_view_conflict_response),
                     confirm_data);
@@ -601,13 +559,13 @@ gimp_action_view_conflict_confirm (GimpActionView  *view,
   gimp_message_box_set_primary_text (box,
                                      _("Shortcut \"%s\" is already taken "
                                        "by \"%s\" from the \"%s\" group."),
-                                     accel_string, stripped, group->label);
+                                     accel_string, label, group->label);
   gimp_message_box_set_text (box,
                              _("Reassigning the shortcut will cause it "
                                "to be removed from \"%s\"."),
-                             stripped);
+                             label);
 
-  g_free (stripped);
+  g_free (label);
   g_free (accel_string);
 
   g_object_unref (group);
@@ -615,13 +573,10 @@ gimp_action_view_conflict_confirm (GimpActionView  *view,
   gtk_widget_show (dialog);
 }
 
-static void
-gimp_action_view_accel_edited (GimpCellRendererAccel *accel,
-                               const char            *path_string,
-                               gboolean               delete,
-                               guint                  accel_key,
-                               GdkModifierType        accel_mask,
-                               GimpActionView        *view)
+static const gchar *
+gimp_action_view_get_accel_action (GimpActionView  *view,
+                                   const gchar     *path_string,
+                                   GtkAction      **action_return)
 {
   GtkTreeModel *model;
   GtkTreePath  *path;
@@ -629,15 +584,13 @@ gimp_action_view_accel_edited (GimpCellRendererAccel *accel,
 
   model = gtk_tree_view_get_model (GTK_TREE_VIEW (view));
   if (! model)
-    return;
+    return NULL;
 
   path = gtk_tree_path_new_from_string (path_string);
 
   if (gtk_tree_model_get_iter (model, &iter, path))
     {
-      GtkAction      *action;
-      GtkActionGroup *group;
-      gchar          *accel_path;
+      GtkAction *action;
 
       gtk_tree_model_get (model, &iter,
                           GIMP_ACTION_VIEW_COLUMN_ACTION, &action,
@@ -646,106 +599,129 @@ gimp_action_view_accel_edited (GimpCellRendererAccel *accel,
       if (! action)
         goto done;
 
-      g_object_get (action, "action-group", &group, NULL);
-
-      if (! group)
-        {
-          g_object_unref (action);
-          goto done;
-        }
-
-#ifdef __GNUC__
-#warning FIXME: remove accel_path hack
-#endif
-      accel_path = g_object_get_data (G_OBJECT (action), "gimp-accel-path");
-
-      if (accel_path)
-        accel_path = g_strdup (accel_path);
-      else
-        accel_path = g_strdup_printf ("<Actions>/%s/%s",
-                                      gtk_action_group_get_name (group),
-                                      gtk_action_get_name (action));
-
-      if (delete)
-        {
-          if (! gtk_accel_map_change_entry (accel_path, 0, 0, FALSE))
-            {
-              g_message (_("Removing shortcut failed."));
-            }
-        }
-      else if (! accel_key)
-        {
-          g_message (_("Invalid shortcut."));
-        }
-      else
-        {
-          if (! gtk_accel_map_change_entry (accel_path,
-                                            accel_key, accel_mask, FALSE))
-            {
-              GtkAction   *conflict_action = NULL;
-              GtkTreeIter  iter;
-              gboolean     iter_valid;
-
-              for (iter_valid = gtk_tree_model_get_iter_first (model, &iter);
-                   iter_valid;
-                   iter_valid = gtk_tree_model_iter_next (model, &iter))
-                {
-                  GtkTreeIter child_iter;
-                  gboolean    child_valid;
-
-                  for (child_valid = gtk_tree_model_iter_children (model,
-                                                                   &child_iter,
-                                                                   &iter);
-                       child_valid;
-                       child_valid = gtk_tree_model_iter_next (model,
-                                                               &child_iter))
-                    {
-                      guint           child_accel_key;
-                      GdkModifierType child_accel_mask;
-
-                      gtk_tree_model_get (model, &child_iter,
-                                          GIMP_ACTION_VIEW_COLUMN_ACCEL_KEY,
-                                          &child_accel_key,
-                                          GIMP_ACTION_VIEW_COLUMN_ACCEL_MASK,
-                                          &child_accel_mask,
-                                          -1);
-
-                      if (accel_key  == child_accel_key &&
-                          accel_mask == child_accel_mask)
-                        {
-                          gtk_tree_model_get (model, &child_iter,
-                                              GIMP_ACTION_VIEW_COLUMN_ACTION,
-                                              &conflict_action,
-                                              -1);
-                          break;
-                        }
-                    }
-
-                  if (conflict_action)
-                    break;
-                }
-
-              if (conflict_action && conflict_action != action)
-                {
-                  gimp_action_view_conflict_confirm (view, conflict_action,
-                                                     accel_key,
-                                                     accel_mask,
-                                                     accel_path);
-                  g_object_unref (conflict_action);
-                }
-              else if (conflict_action != action)
-                {
-                  g_message (_("Changing shortcut failed."));
-                }
-            }
-        }
-
-      g_free (accel_path);
-      g_object_unref (group);
+      gtk_tree_path_free (path);
       g_object_unref (action);
+
+      *action_return = action;
+
+      return gtk_action_get_accel_path (action);
     }
 
  done:
-
   gtk_tree_path_free (path);
+
+  return NULL;
+}
+
+static void
+gimp_action_view_accel_edited (GtkCellRendererAccel *accel,
+                               const char           *path_string,
+                               guint                 accel_key,
+                               GdkModifierType       accel_mask,
+                               guint                 hardware_keycode,
+                               GimpActionView       *view)
+{
+  GtkAction   *action;
+  const gchar *accel_path;
+
+  accel_path = gimp_action_view_get_accel_action (view, path_string,
+                                                  &action);
+
+  if (! accel_path)
+    return;
+
+  if (! accel_key)
+    {
+      gimp_message (view->manager->gimp, G_OBJECT (view),
+                    GIMP_MESSAGE_ERROR,
+                    _("Invalid shortcut."));
+    }
+  else if (! gtk_accel_map_change_entry (accel_path,
+                                         accel_key, accel_mask, FALSE))
+    {
+      GtkTreeModel *model;
+      GtkAction    *conflict_action = NULL;
+      GtkTreeIter   iter;
+      gboolean      iter_valid;
+
+      model = gtk_tree_view_get_model (GTK_TREE_VIEW (view));
+
+      for (iter_valid = gtk_tree_model_get_iter_first (model, &iter);
+           iter_valid;
+           iter_valid = gtk_tree_model_iter_next (model, &iter))
+        {
+          GtkTreeIter child_iter;
+          gboolean    child_valid;
+
+          for (child_valid = gtk_tree_model_iter_children (model,
+                                                           &child_iter,
+                                                           &iter);
+               child_valid;
+               child_valid = gtk_tree_model_iter_next (model, &child_iter))
+            {
+              guint           child_accel_key;
+              GdkModifierType child_accel_mask;
+
+              gtk_tree_model_get (model, &child_iter,
+                                  GIMP_ACTION_VIEW_COLUMN_ACCEL_KEY,
+                                  &child_accel_key,
+                                  GIMP_ACTION_VIEW_COLUMN_ACCEL_MASK,
+                                  &child_accel_mask,
+                                  -1);
+
+              if (accel_key  == child_accel_key &&
+                  accel_mask == child_accel_mask)
+                {
+                  gtk_tree_model_get (model, &child_iter,
+                                      GIMP_ACTION_VIEW_COLUMN_ACTION,
+                                      &conflict_action,
+                                      -1);
+                  break;
+                }
+            }
+
+          if (conflict_action)
+            break;
+        }
+
+      if (conflict_action != action)
+        {
+          if (conflict_action)
+            {
+              gimp_action_view_conflict_confirm (view, conflict_action,
+                                                 accel_key,
+                                                 accel_mask,
+                                                 accel_path);
+              g_object_unref (conflict_action);
+            }
+          else
+            {
+              gimp_message (view->manager->gimp, G_OBJECT (view),
+                            GIMP_MESSAGE_ERROR,
+                            _("Changing shortcut failed."));
+            }
+        }
+    }
+}
+
+static void
+gimp_action_view_accel_cleared (GtkCellRendererAccel *accel,
+                                const char           *path_string,
+                                GimpActionView       *view)
+{
+  GtkAction   *action;
+  const gchar *accel_path;
+
+  accel_path = gimp_action_view_get_accel_action (view, path_string,
+                                                  &action);
+
+  if (! accel_path)
+    return;
+
+  if (! gtk_accel_map_change_entry (accel_path, 0, 0, FALSE))
+    {
+      gimp_message (view->manager->gimp, G_OBJECT (view),
+                    GIMP_MESSAGE_ERROR,
+                    _("Removing shortcut failed."));
+    }
 }
