@@ -1,4 +1,4 @@
-/* The GIMP -- an image manipulation program
+/* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
  *
  * gimpfiledialog.c
@@ -25,6 +25,7 @@
 
 #include <gtk/gtk.h>
 
+#include "libgimpbase/gimpbase.h"
 #include "libgimpwidgets/gimpwidgets.h"
 
 #include "widgets-types.h"
@@ -33,17 +34,19 @@
 #include "core/gimpimage.h"
 #include "core/gimpprogress.h"
 
-#include "config/gimpcoreconfig.h"
 #include "config/gimpguiconfig.h"
 
 #include "file/file-utils.h"
 
-#include "plug-in/plug-in-proc-def.h"
-#include "plug-in/plug-ins.h"
+#include "pdb/gimppdb.h"
+
+#include "plug-in/gimppluginmanager.h"
+#include "plug-in/gimppluginprocedure.h"
 
 #include "gimpfiledialog.h"
 #include "gimpfileprocview.h"
 #include "gimphelp-ids.h"
+#include "gimpprogressbox.h"
 #include "gimpview.h"
 #include "gimpviewrendererimagefile.h"
 #include "gimpthumbbox.h"
@@ -52,8 +55,7 @@
 #include "gimp-intl.h"
 
 
-static void  gimp_file_dialog_class_init            (GimpFileDialogClass   *klass);
-static void  gimp_file_dialog_progress_iface_init   (GimpProgressInterface *progress_iface);
+static void    gimp_file_dialog_progress_iface_init (GimpProgressInterface *iface);
 static gboolean gimp_file_dialog_delete_event       (GtkWidget        *widget,
                                                      GdkEventAny      *event);
 static void     gimp_file_dialog_response           (GtkDialog        *dialog,
@@ -70,6 +72,8 @@ static void     gimp_file_dialog_progress_set_text  (GimpProgress     *progress,
 static void     gimp_file_dialog_progress_set_value (GimpProgress     *progress,
                                                      gdouble           percentage);
 static gdouble  gimp_file_dialog_progress_get_value (GimpProgress     *progress);
+static void     gimp_file_dialog_progress_pulse     (GimpProgress     *progress);
+static guint32  gimp_file_dialog_progress_get_window(GimpProgress     *progress);
 
 static void     gimp_file_dialog_add_preview        (GimpFileDialog   *dialog,
                                                      Gimp             *gimp);
@@ -98,43 +102,13 @@ static void     gimp_file_dialog_help_clicked       (GtkWidget        *widget,
 static gchar  * gimp_file_dialog_pattern_from_extension (const gchar  *extension);
 
 
-GType
-gimp_file_dialog_get_type (void)
-{
-  static GType dialog_type = 0;
+G_DEFINE_TYPE_WITH_CODE (GimpFileDialog, gimp_file_dialog,
+                         GTK_TYPE_FILE_CHOOSER_DIALOG,
+                         G_IMPLEMENT_INTERFACE (GIMP_TYPE_PROGRESS,
+                                                gimp_file_dialog_progress_iface_init))
 
-  if (! dialog_type)
-    {
-      static const GTypeInfo dialog_info =
-      {
-        sizeof (GimpFileDialogClass),
-        (GBaseInitFunc) NULL,
-        (GBaseFinalizeFunc) NULL,
-        (GClassInitFunc) gimp_file_dialog_class_init,
-        NULL,           /* class_finalize */
-        NULL,           /* class_data     */
-        sizeof (GimpFileDialog),
-        0,              /* n_preallocs    */
-        NULL,           /* instance_init  */
-      };
+#define parent_class gimp_file_dialog_parent_class
 
-      static const GInterfaceInfo progress_iface_info =
-      {
-        (GInterfaceInitFunc) gimp_file_dialog_progress_iface_init,
-        NULL,           /* iface_finalize */
-        NULL            /* iface_data     */
-      };
-
-      dialog_type = g_type_register_static (GTK_TYPE_FILE_CHOOSER_DIALOG,
-                                            "GimpFileDialog",
-                                            &dialog_info, 0);
-
-      g_type_add_interface_static (dialog_type, GIMP_TYPE_PROGRESS,
-                                   &progress_iface_info);
-    }
-
-  return dialog_type;
-}
 
 static void
 gimp_file_dialog_class_init (GimpFileDialogClass *klass)
@@ -148,14 +122,21 @@ gimp_file_dialog_class_init (GimpFileDialogClass *klass)
 }
 
 static void
-gimp_file_dialog_progress_iface_init (GimpProgressInterface *progress_iface)
+gimp_file_dialog_init (GimpFileDialog *dialog)
 {
-  progress_iface->start     = gimp_file_dialog_progress_start;
-  progress_iface->end       = gimp_file_dialog_progress_end;
-  progress_iface->is_active = gimp_file_dialog_progress_is_active;
-  progress_iface->set_text  = gimp_file_dialog_progress_set_text;
-  progress_iface->set_value = gimp_file_dialog_progress_set_value;
-  progress_iface->get_value = gimp_file_dialog_progress_get_value;
+}
+
+static void
+gimp_file_dialog_progress_iface_init (GimpProgressInterface *iface)
+{
+  iface->start      = gimp_file_dialog_progress_start;
+  iface->end        = gimp_file_dialog_progress_end;
+  iface->is_active  = gimp_file_dialog_progress_is_active;
+  iface->set_text   = gimp_file_dialog_progress_set_text;
+  iface->set_value  = gimp_file_dialog_progress_set_value;
+  iface->get_value  = gimp_file_dialog_progress_get_value;
+  iface->pulse      = gimp_file_dialog_progress_pulse;
+  iface->get_window = gimp_file_dialog_progress_get_window;
 }
 
 static gboolean
@@ -175,8 +156,11 @@ gimp_file_dialog_response (GtkDialog *dialog,
     {
       file_dialog->canceled = TRUE;
 
-      if (file_dialog->progress_active && file_dialog->progress_cancelable)
-        gimp_progress_cancel (GIMP_PROGRESS (dialog));
+      if (GIMP_PROGRESS_BOX (file_dialog->progress)->active &&
+          GIMP_PROGRESS_BOX (file_dialog->progress)->cancelable)
+        {
+          gimp_progress_cancel (GIMP_PROGRESS (dialog));
+        }
     }
 }
 
@@ -186,23 +170,16 @@ gimp_file_dialog_progress_start (GimpProgress *progress,
                                  gboolean      cancelable)
 {
   GimpFileDialog *dialog = GIMP_FILE_DIALOG (progress);
+  GimpProgress   *retval;
 
-  if (! dialog->progress_active)
-    {
-      GtkProgressBar *bar = GTK_PROGRESS_BAR (dialog->progress);
+  retval = gimp_progress_start (GIMP_PROGRESS (dialog->progress),
+                                message, cancelable);
+  gtk_widget_show (dialog->progress);
 
-      gtk_progress_bar_set_text (bar, message);
-      gtk_progress_bar_set_fraction (bar, 0.0);
+  gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog),
+                                     GTK_RESPONSE_CANCEL, cancelable);
 
-      gtk_widget_show (dialog->progress);
-
-      dialog->progress_active     = TRUE;
-      dialog->progress_cancelable = cancelable;
-
-      return progress;
-    }
-
-  return NULL;
+  return retval;
 }
 
 static void
@@ -210,18 +187,8 @@ gimp_file_dialog_progress_end (GimpProgress *progress)
 {
   GimpFileDialog *dialog = GIMP_FILE_DIALOG (progress);
 
-  if (dialog->progress_active)
-    {
-      GtkProgressBar *bar = GTK_PROGRESS_BAR (dialog->progress);
-
-      gtk_progress_bar_set_text (bar, "");
-      gtk_progress_bar_set_fraction (bar, 0.0);
-
-      gtk_widget_hide (dialog->progress);
-
-      dialog->progress_active     = FALSE;
-      dialog->progress_cancelable = FALSE;
-    }
+  gimp_progress_end (GIMP_PROGRESS (dialog->progress));
+  gtk_widget_hide (dialog->progress);
 }
 
 static gboolean
@@ -229,7 +196,7 @@ gimp_file_dialog_progress_is_active (GimpProgress *progress)
 {
   GimpFileDialog *dialog = GIMP_FILE_DIALOG (progress);
 
-  return dialog->progress_active;
+  return gimp_progress_is_active (GIMP_PROGRESS (dialog->progress));
 }
 
 static void
@@ -238,12 +205,7 @@ gimp_file_dialog_progress_set_text (GimpProgress *progress,
 {
   GimpFileDialog *dialog = GIMP_FILE_DIALOG (progress);
 
-  if (dialog->progress_active)
-    {
-      GtkProgressBar *bar = GTK_PROGRESS_BAR (dialog->progress);
-
-      gtk_progress_bar_set_text (bar, message);
-    }
+  gimp_progress_set_text (GIMP_PROGRESS (dialog->progress), message);
 }
 
 static void
@@ -252,12 +214,7 @@ gimp_file_dialog_progress_set_value (GimpProgress *progress,
 {
   GimpFileDialog *dialog = GIMP_FILE_DIALOG (progress);
 
-  if (dialog->progress_active)
-    {
-      GtkProgressBar *bar = GTK_PROGRESS_BAR (dialog->progress);
-
-      gtk_progress_bar_set_fraction (bar, percentage);
-    }
+  gimp_progress_set_value (GIMP_PROGRESS (dialog->progress), percentage);
 }
 
 static gdouble
@@ -265,14 +222,23 @@ gimp_file_dialog_progress_get_value (GimpProgress *progress)
 {
   GimpFileDialog *dialog = GIMP_FILE_DIALOG (progress);
 
-  if (dialog->progress_active)
-    {
-      GtkProgressBar *bar = GTK_PROGRESS_BAR (dialog->progress);
+  return gimp_progress_get_value (GIMP_PROGRESS (dialog->progress));
+}
 
-      return gtk_progress_bar_get_fraction (bar);
-    }
+static void
+gimp_file_dialog_progress_pulse (GimpProgress *progress)
+{
+  GimpFileDialog *dialog = GIMP_FILE_DIALOG (progress);
 
-  return 0.0;
+  gimp_progress_pulse (GIMP_PROGRESS (dialog->progress));
+}
+
+static guint32
+gimp_file_dialog_progress_get_window (GimpProgress *progress)
+{
+  GimpFileDialog *dialog = GIMP_FILE_DIALOG (progress);
+
+  return (guint32) gimp_window_get_native (GTK_WINDOW (dialog));
 }
 
 
@@ -290,6 +256,8 @@ gimp_file_dialog_new (Gimp                 *gimp,
   GSList         *file_procs;
   const gchar    *automatic;
   const gchar    *automatic_help_id;
+  const gchar    *pictures;
+  gboolean        local_only;
 
   g_return_val_if_fail (GIMP_IS_GIMP (gimp), NULL);
   g_return_val_if_fail (title != NULL, NULL);
@@ -300,15 +268,23 @@ gimp_file_dialog_new (Gimp                 *gimp,
   switch (action)
     {
     case GTK_FILE_CHOOSER_ACTION_OPEN:
-      file_procs = gimp->load_procs;
+      file_procs = gimp->plug_in_manager->load_procs;
       automatic  = _("Automatically Detected");
       automatic_help_id = GIMP_HELP_FILE_OPEN_BY_EXTENSION;
+
+      /* FIXME */
+      local_only = (gimp_pdb_lookup_procedure (gimp->pdb,
+                                               "file-uri-load") == NULL);
       break;
 
     case GTK_FILE_CHOOSER_ACTION_SAVE:
-      file_procs = gimp->save_procs;
+      file_procs = gimp->plug_in_manager->save_procs;
       automatic  = _("By Extension");
       automatic_help_id = GIMP_HELP_FILE_SAVE_BY_EXTENSION;
+
+      /* FIXME */
+      local_only = (gimp_pdb_lookup_procedure (gimp->pdb,
+                                               "file-uri-save") == NULL);
       break;
 
     default:
@@ -317,9 +293,11 @@ gimp_file_dialog_new (Gimp                 *gimp,
     }
 
   dialog = g_object_new (GIMP_TYPE_FILE_DIALOG,
-                         "title",  title,
-                         "role",   role,
-                         "action", action,
+                         "title",                     title,
+                         "role",                      role,
+                         "action",                    action,
+                         "local-only",                local_only,
+                         "do-overwrite-confirmation", TRUE,
                          NULL);
 
   gtk_dialog_add_buttons (GTK_DIALOG (dialog),
@@ -328,6 +306,10 @@ gimp_file_dialog_new (Gimp                 *gimp,
                           NULL);
 
   gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
+  gtk_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
+                                           GTK_RESPONSE_OK,
+                                           GTK_RESPONSE_CANCEL,
+                                           -1);
 
   gimp_help_connect (GTK_WIDGET (dialog),
                      gimp_file_dialog_help_func, help_id, dialog);
@@ -349,8 +331,16 @@ gimp_file_dialog_new (Gimp                 *gimp,
       g_signal_connect (button, "clicked",
                         G_CALLBACK (gimp_file_dialog_help_clicked),
                         dialog);
+
+      g_object_set_data (G_OBJECT (dialog), "gimp-dialog-help-button", button);
     }
 
+  pictures = gimp_user_directory (GIMP_USER_DIRECTORY_PICTURES);
+
+  if (pictures)
+    gtk_file_chooser_add_shortcut_folder (GTK_FILE_CHOOSER (dialog),
+                                          pictures, NULL);
+
   gimp_file_dialog_add_preview (dialog, gimp);
 
   gimp_file_dialog_add_filters (dialog, gimp, file_procs);
@@ -358,7 +348,7 @@ gimp_file_dialog_new (Gimp                 *gimp,
   gimp_file_dialog_add_proc_selection (dialog, gimp, file_procs, automatic,
                                        automatic_help_id);
 
-  dialog->progress = gtk_progress_bar_new ();
+  dialog->progress = gimp_progress_box_new ();
   gtk_box_pack_end (GTK_BOX (GTK_DIALOG (dialog)->vbox), dialog->progress,
                     FALSE, FALSE, 0);
 
@@ -369,17 +359,37 @@ void
 gimp_file_dialog_set_sensitive (GimpFileDialog *dialog,
                                 gboolean        sensitive)
 {
+  GList *children;
+  GList *list;
+
   g_return_if_fail (GIMP_IS_FILE_DIALOG (dialog));
 
-  gimp_dialog_set_sensitive (GTK_DIALOG (dialog), sensitive);
+  children =
+    gtk_container_get_children (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox));
+
+  for (list = children; list; list = g_list_next (list))
+    {
+      /*  skip the last item (the action area) */
+      if (! g_list_next (list))
+        break;
+
+      gtk_widget_set_sensitive (list->data, sensitive);
+    }
+
+  g_list_free (children);
+
+  gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog),
+                                     GTK_RESPONSE_CANCEL, sensitive);
+  gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog),
+                                     GTK_RESPONSE_OK, sensitive);
 
   dialog->busy     = ! sensitive;
   dialog->canceled = FALSE;
 }
 
 void
-gimp_file_dialog_set_file_proc (GimpFileDialog *dialog,
-                                PlugInProcDef  *file_proc)
+gimp_file_dialog_set_file_proc (GimpFileDialog      *dialog,
+                                GimpPlugInProcedure *file_proc)
 {
   g_return_if_fail (GIMP_IS_FILE_DIALOG (dialog));
 
@@ -390,37 +400,94 @@ gimp_file_dialog_set_file_proc (GimpFileDialog *dialog,
 
 void
 gimp_file_dialog_set_image (GimpFileDialog *dialog,
-                            GimpImage      *gimage,
-                            gboolean        save_a_copy)
+                            GimpImage      *image,
+                            gboolean        save_a_copy,
+                            gboolean        close_after_saving)
 {
   const gchar *uri = NULL;
   gchar       *dirname;
   gchar       *basename;
 
   g_return_if_fail (GIMP_IS_FILE_DIALOG (dialog));
-  g_return_if_fail (GIMP_IS_IMAGE (gimage));
+  g_return_if_fail (GIMP_IS_IMAGE (image));
 
-  dialog->gimage      = gimage;
-  dialog->save_a_copy = save_a_copy;
+  dialog->image              = image;
+  dialog->save_a_copy        = save_a_copy;
+  dialog->close_after_saving = close_after_saving;
 
   if (save_a_copy)
-    uri = g_object_get_data (G_OBJECT (gimage), "gimp-image-save-a-copy");
+    uri = g_object_get_data (G_OBJECT (image), "gimp-image-save-a-copy");
 
   if (! uri)
-    uri = gimp_image_get_uri (gimage);
+    uri = gimp_image_get_uri (image);
 
   gimp_file_dialog_set_file_proc (dialog, NULL);
 
+#ifndef G_OS_WIN32
   dirname  = g_path_get_dirname (uri);
-  basename = g_path_get_basename (uri);
+#else
+  /* g_path_get_dirname() is supposed to work on pathnames, not URIs.
+   *
+   * If uri points to a file on the root of a drive
+   * "file:///d:/foo.png", g_path_get_dirname() would return
+   * "file:///d:". (What we really would want is "file:///d:/".) When
+   * this then is passed inside gtk+ to g_filename_from_uri() we get
+   * "d:" which is not an absolute pathname. This currently causes an
+   * assertion failure in gtk+. This scenario occurs if we have opened
+   * an image from the root of a drive and then do Save As.
+   *
+   * Of course, gtk+ shouldn't assert even if we feed it slighly bogus
+   * data, and that problem should be fixed, too. But to get the
+   * correct default current folder in the filechooser combo box, we
+   * need to pass it the proper URI for an absolute path anyway. So
+   * don't use g_path_get_dirname() on file: URIs.
+   */
+  if (g_str_has_prefix (uri, "file:///"))
+    {
+      gchar *filepath = g_filename_from_uri (uri, NULL, NULL);
+      gchar *dirpath  = NULL;
+
+      if (filepath != NULL)
+	{
+	  dirpath = g_path_get_dirname (filepath);
+	  g_free (filepath);
+	}
+
+      if (dirpath != NULL)
+	{
+	  dirname = g_filename_to_uri (dirpath, NULL, NULL);
+	  g_free (dirpath);
+	}
+      else
+        {
+          dirname = NULL;
+        }
+    }
+  else
+    {
+      dirname = g_path_get_dirname (uri);
+    }
+#endif
 
   if (dirname && strlen (dirname) && strcmp (dirname, "."))
-    gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (dialog),
-                                             dirname);
+    {
+      gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (dialog),
+                                               dirname);
+    }
+  else
+    {
+      const gchar *folder;
 
-  gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog), basename);
+      folder = g_object_get_data (G_OBJECT (image), "gimp-image-dirname");
+
+      if (folder)
+        gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), folder);
+    }
 
   g_free (dirname);
+
+  basename = file_utils_uri_display_basename (uri);
+  gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog), basename);
   g_free (basename);
 }
 
@@ -443,7 +510,7 @@ gimp_file_dialog_add_preview (GimpFileDialog *dialog,
                     G_CALLBACK (gimp_file_dialog_update_preview),
                     dialog);
 
-  dialog->thumb_box = gimp_thumb_box_new (gimp);
+  dialog->thumb_box = gimp_thumb_box_new (gimp_get_user_context (gimp));
   gtk_widget_set_sensitive (GTK_WIDGET (dialog->thumb_box), FALSE);
   gtk_file_chooser_set_preview_widget (GTK_FILE_CHOOSER (dialog),
                                        dialog->thumb_box);
@@ -459,35 +526,30 @@ gimp_file_dialog_add_filters (GimpFileDialog *dialog,
                               Gimp           *gimp,
                               GSList         *file_procs)
 {
-  GtkFileFilter *filter;
+  GtkFileFilter *all;
   GSList        *list;
 
-  filter = gtk_file_filter_new ();
-  gtk_file_filter_set_name (filter, _("All Files"));
-  gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter);
-  gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (dialog), filter);
-  gtk_file_filter_add_pattern (filter, "*");
+  all = gtk_file_filter_new ();
+  gtk_file_filter_set_name (all, _("All files"));
+  gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), all);
+  gtk_file_filter_add_pattern (all, "*");
+
+  all = gtk_file_filter_new ();
+  gtk_file_filter_set_name (all, _("All images"));
+  gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), all);
 
   for (list = file_procs; list; list = g_slist_next (list))
     {
-      PlugInProcDef *file_proc = list->data;
+      GimpPlugInProcedure *file_proc = list->data;
 
       if (file_proc->extensions_list)
         {
-          const gchar *domain;
-          gchar       *label;
-          GString     *str;
-          GSList      *ext;
-          gint         i;
+          GtkFileFilter *filter = gtk_file_filter_new ();
+          GString       *str;
+          GSList        *ext;
+          gint           i;
 
-          domain = plug_ins_locale_domain (gimp, file_proc->prog, NULL);
-
-          label = plug_in_proc_def_get_label (file_proc, domain);
-
-          str = g_string_new (label);
-          g_free (label);
-
-          filter = gtk_file_filter_new ();
+          str = g_string_new (gimp_plug_in_procedure_get_label (file_proc));
 
 /*  an arbitrary limit to keep the file dialog from becoming too wide  */
 #define MAX_EXTENSIONS  4
@@ -501,6 +563,7 @@ gimp_file_dialog_add_filters (GimpFileDialog *dialog,
 
               pattern = gimp_file_dialog_pattern_from_extension (extension);
               gtk_file_filter_add_pattern (filter, pattern);
+              gtk_file_filter_add_pattern (all, pattern);
               g_free (pattern);
 
               if (i == 0)
@@ -534,6 +597,8 @@ gimp_file_dialog_add_filters (GimpFileDialog *dialog,
           gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter);
         }
     }
+
+  gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (dialog), all);
 }
 
 static void
@@ -584,11 +649,8 @@ static void
 gimp_file_dialog_update_preview (GtkFileChooser *chooser,
                                  GimpFileDialog *dialog)
 {
-  gchar *uri = gtk_file_chooser_get_preview_uri (chooser);
-
-  gimp_thumb_box_set_uri (GIMP_THUMB_BOX (dialog->thumb_box), uri);
-
-  g_free (uri);
+  gimp_thumb_box_take_uri (GIMP_THUMB_BOX (dialog->thumb_box),
+                           gtk_file_chooser_get_preview_uri (chooser));
 }
 
 static void
@@ -612,7 +674,7 @@ gimp_file_dialog_proc_changed (GimpFileProcView *view,
 
   if (gtk_file_chooser_get_action (chooser) == GTK_FILE_CHOOSER_ACTION_SAVE)
     {
-      PlugInProcDef *proc = dialog->file_proc;
+      GimpPlugInProcedure *proc = dialog->file_proc;
 
       if (proc && proc->extensions_list)
         {
@@ -668,7 +730,7 @@ gimp_file_dialog_proc_changed (GimpFileProcView *view,
 
                   gtk_file_chooser_set_uri (chooser, s->str);
 
-                  basename = file_utils_uri_to_utf8_basename (s->str);
+                  basename = file_utils_uri_display_basename (s->str);
                   gtk_file_chooser_set_current_name (chooser, basename);
                   g_free (basename);
 
