@@ -29,21 +29,21 @@ typedef enum
 
 typedef struct
 {
-  gdouble ambient_int;
-  gdouble diffuse_int;
-  gdouble diffuse_ref;
-  gdouble specular_ref;
-  gdouble highlight;
-  GckRGB  color;
+  gdouble  ambient_int;
+  gdouble  diffuse_int;
+  gdouble  diffuse_ref;
+  gdouble  specular_ref;
+  gdouble  highlight;
+  GimpRGB  color;
 } MaterialSettings;
 
 typedef struct
 {
-  LightType   type;
-  GimpVector3 position;
-  GimpVector3 direction;
-  GckRGB      color;
-  gdouble     intensity;
+  LightType    type;
+  GimpVector3  position;
+  GimpVector3  direction;
+  GimpRGB      color;
+  gdouble      intensity;
 } LightSettings;
 
 typedef struct
@@ -61,11 +61,9 @@ typedef struct
   gint transparent_background;
   gint tiled;
   gint showgrid;
-  gint tooltips_enabled;
   gint showcaps;
 
-  glong preview_zoom_factor;
-
+  gdouble zoom;
   gdouble alpha,beta,gamma;
   gdouble maxdepth;
   gdouble pixeltreshold;
@@ -75,13 +73,12 @@ typedef struct
 
   gint32 boxmap_id[6];
   gint32 cylindermap_id[2];
-  
+
 } MapObjectValues;
 
 /* Externally visible variables */
 /* ============================ */
 
 extern MapObjectValues mapvals;
-extern GckRGB background;
 
 #endif  /* __MAPOBJECT_MAIN_H__ */
